﻿using System;
using System.Globalization;
using System.Windows.Data;

namespace MadMediaManager.Converters
{
    [ValueConversion(typeof(object), typeof(string))]
    public class FormattingConverter : IValueConverter
    {
        /// <summary>
        /// Converts a value. 
        /// </summary>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        /// <param name="value">The value produced by the binding source.</param><param name="targetType">The type of the binding target property.</param><param name="parameter">The converter parameter to use.</param><param name="culture">The culture to use in the converter.</param>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value == null)
            {
                return null;
            }

            if (parameter != null)
            {
                var formatString = parameter.ToString();

                if (!string.IsNullOrEmpty(formatString))
                {
                    return string.Format(culture, formatString, value);
                }
            }

            return value.ToString();
        }

        /// <summary>
        /// Converts a value. 
        /// </summary>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        /// <param name="value">The value that is produced by the binding target.</param><param name="targetType">The type to convert to.</param><param name="parameter">The converter parameter to use.</param><param name="culture">The culture to use in the converter.</param>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var typeConverter = System.ComponentModel.TypeDescriptor.GetConverter(targetType);

            return typeConverter.CanConvertFrom(value.GetType()) ? typeConverter.ConvertFrom(value) : value;
        }
    }
}
